@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -78,9 +78,9 @@ import com.openexchange.jsieve.commands.TestCommand;
 /**
  * This class is used to convert the JJTree Objects into the internal
  * representation via the visitor pattern
- * 
+ *
  * @author d7
- * 
+ *
  */
 public class InternalVisitor implements SieveParserVisitor {
 
@@ -145,6 +145,7 @@ public class InternalVisitor implements SieveParserVisitor {
                         ifCommand.setTestcommand(command);
                     } else if (jjtAccept instanceof ArrayList) {
                         final ArrayList command = (ArrayList) jjtAccept;
+                        checkArrayforActionCommands(command);
                         ifCommand.setActioncommands(command);
                     }
                 }
@@ -371,4 +372,12 @@ public class InternalVisitor implements SieveParserVisitor {
         return list;
     }
 
+    private void checkArrayforActionCommands(ArrayList<?> command) throws SieveException {
+        for (final Object obj : command) {
+            if (!(obj instanceof ActionCommand)) {
+                throw new SieveException("Found nested rule, instead of ActionCommand. This is not allowed in this implementation.");
+            }
+        }
+    }
+
 }
