@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -50,32 +50,43 @@
 package com.openexchange.mailfilter.ajax;
 
 import java.io.IOException;
+import javax.servlet.ServletConfig;
+import javax.servlet.ServletException;
 import javax.servlet.http.HttpServlet;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 import org.apache.commons.logging.Log;
 import org.apache.commons.logging.LogFactory;
 import org.json.JSONException;
+import com.openexchange.ajax.SessionServlet;
 import com.openexchange.ajax.container.Response;
 import com.openexchange.ajax.writer.ResponseWriter;
+import com.openexchange.configuration.CookieHashSource;
+import com.openexchange.configuration.ServerConfig.Property;
 import com.openexchange.groupware.AbstractOXException;
-import com.openexchange.mailfilter.ajax.SessionWrapper.Credentials;
 import com.openexchange.mailfilter.ajax.actions.AbstractAction;
 import com.openexchange.mailfilter.ajax.actions.AbstractRequest;
 import com.openexchange.mailfilter.ajax.exceptions.OXMailfilterException;
-import com.openexchange.sessiond.exception.SessiondException;
+import com.openexchange.mailfilter.services.MailFilterServletServiceRegistry;
+import com.openexchange.server.ServiceException;
+import com.openexchange.session.Session;
+import com.openexchange.sessiond.SessionExceptionCodes;
+import com.openexchange.sessiond.SessiondException;
+import com.openexchange.sessiond.SessiondService;
 import com.openexchange.tools.servlet.AjaxException;
 import com.openexchange.tools.servlet.http.Tools;
 
 /**
- * 
+ *
  * @author <a href="mailto:marcus@open-xchange.org">Marcus Klein</a>
  */
 public abstract class AJAXServlet extends HttpServlet {
 
 	private static final long serialVersionUID = 3006497622205429579L;
 
-    private static final Log LOG = LogFactory.getLog(AJAXServlet.class);
+    private static final Log LOG = com.openexchange.log.Log.valueOf(LogFactory.getLog(AJAXServlet.class));
+
+    private static final String PARAMETER_SESSION = com.openexchange.ajax.AJAXServlet.PARAMETER_SESSION;
 
     /**
      * The content type if the response body contains javascript data. Set it
@@ -83,39 +94,54 @@ public abstract class AJAXServlet extends HttpServlet {
      */
     public static final String CONTENTTYPE_JAVASCRIPT = "text/javascript; charset=UTF-8";
 
+    private CookieHashSource hashSource;
+
     protected AJAXServlet() {
         super();
     }
 
+    @Override
+    public void init(final ServletConfig config) throws ServletException {
+        super.init(config);
+        hashSource = CookieHashSource.parse(config.getInitParameter(Property.COOKIE_HASH.getPropertyName()));
+    }
+
     protected static void sendError(final HttpServletResponse resp) throws IOException {
         resp.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
     }
 
-    protected static SessionWrapper getSession(final HttpServletRequest req) throws SessiondException, OXMailfilterException {
-        return new SessionWrapper(req);
-    }
-
     @Override
     protected void doGet(final HttpServletRequest req, final HttpServletResponse resp) throws IOException {
         final Response response = new Response();
         try {
+            final String sessionId = req.getParameter(PARAMETER_SESSION);
+            if (sessionId == null) {
+                throw new OXMailfilterException(OXMailfilterException.Code.MISSING_PARAMETER, "session");
+            }
+
+            final SessiondService service = MailFilterServletServiceRegistry.getServiceRegistry().getService(SessiondService.class);
+            if (null == service) {
+                throw new SessiondException(new ServiceException(ServiceException.Code.SERVICE_UNAVAILABLE));
+            }
+            final Session session = service.getSession(sessionId);
+            if (null == session) {
+                if (LOG.isInfoEnabled()) {
+                    LOG.info("There is no session associated with session identifier: " + sessionId);
+                }
+                throw SessionExceptionCodes.SESSION_EXPIRED.create(sessionId);
+            }
+            final String secret = SessionServlet.extractSecret(hashSource, req, session.getHash(), session.getClient());
+            // Check if session is valid
+            if (!session.getSecret().equals(secret)) {
+                if (LOG.isInfoEnabled() && null != secret) {
+                    LOG.info("Session secret is different. Given secret \"" + secret + "\" differs from secret in session \"" + session.getSecret() + "\".");
+                }
+                throw SessionExceptionCodes.WRONG_SESSION_SECRET.create();
+            }
+
             final AbstractRequest request = createRequest();
-            request.setSession(new AbstractRequest.Session() {
-                final SessionWrapper session = getSession(req);
-                
-                public Object getAttribute(final String name) {
-                    return session.getParameter(name);
-                }
-                public void setAttribute(final String name, final Object value) {
-                    session.setParameter(name, value);
-                }
-                public void removeAttribute(final String name) {
-                    session.removeParameter(name);
-                }
-                public Credentials getCredentails() {
-                    return session.getCredentials();
-                }
-            });
+            request.setSession(session);
+
             request.setParameters(new AbstractRequest.Parameters() {
                 public String getParameter(final Parameter param) throws AjaxException {
                     final String value = req.getParameter(param.getName());
@@ -128,7 +154,7 @@ public abstract class AJAXServlet extends HttpServlet {
             /*
              * A non-download action
              */
-            final AbstractAction action = createAction();
+            final AbstractAction action = createAction(session);
             response.setData(action.action(request));
         } catch (final AbstractOXException e) {
             LOG.error(e.getMessage(), e);
@@ -155,26 +181,30 @@ public abstract class AJAXServlet extends HttpServlet {
     protected void doPut(final HttpServletRequest req, final HttpServletResponse resp) throws IOException {
         final Response response = new Response();
         try {
+            final String sessionId = req.getParameter(PARAMETER_SESSION);
+            final SessiondService service = MailFilterServletServiceRegistry.getServiceRegistry().getService(SessiondService.class);
+            if (null == service) {
+                throw new SessiondException(new ServiceException(ServiceException.Code.SERVICE_UNAVAILABLE));
+            }
+            final Session session = service.getSession(sessionId);
+            if (null == session) {
+                if (LOG.isInfoEnabled()) {
+                    LOG.info("There is no session associated with session identifier: " + sessionId);
+                }
+                throw SessionExceptionCodes.SESSION_EXPIRED.create(sessionId);
+            }
+            final String secret = SessionServlet.extractSecret(hashSource, req, session.getHash(), session.getClient());
+            // Check if session is valid
+            if (!session.getSecret().equals(secret)) {
+                if (LOG.isInfoEnabled() && null != secret) {
+                    LOG.info("Session secret is different. Given secret \"" + secret + "\" differs from secret in session \"" + session.getSecret() + "\".");
+                }
+                throw SessionExceptionCodes.WRONG_SESSION_SECRET.create();
+            }
+
             final AbstractRequest request = createRequest();
-            request.setSession(new AbstractRequest.Session() {
-                final SessionWrapper session = getSession(req);
-                
-                public Object getAttribute(final String name) {
-                    return session.getParameter(name);
-                }
+            request.setSession(session);
 
-                public void setAttribute(final String name, final Object value) {
-                    session.setParameter(name, value);
-                }
-
-                public void removeAttribute(final String name) {
-                    session.removeParameter(name);
-                }
-
-                public Credentials getCredentails() {
-                    return session.getCredentials();
-                }
-            });
             request.setParameters(new AbstractRequest.Parameters() {
                 public String getParameter(final Parameter param) throws AjaxException {
                     final String value = req.getParameter(param.getName());
@@ -185,7 +215,7 @@ public abstract class AJAXServlet extends HttpServlet {
                 }
             });
             request.setBody(com.openexchange.ajax.AJAXServlet.getBody(req));
-            final AbstractAction action = createAction();
+            final AbstractAction action = createAction(session);
             response.setData(action.action(request));
         } catch (final AbstractOXException e) {
             LOG.error(e.getMessage(), e);
@@ -207,5 +237,5 @@ public abstract class AJAXServlet extends HttpServlet {
 
     protected abstract AbstractRequest createRequest();
 
-    protected abstract AbstractAction<?, ? extends AbstractRequest> createAction();
+    protected abstract AbstractAction<?, ? extends AbstractRequest> createAction(Session session);
 }
