@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -49,21 +49,23 @@
 
 package com.openexchange.mailfilter.ajax.actions;
 
+import com.openexchange.config.ConfigurationService;
 import com.openexchange.mailfilter.ajax.Action;
+import com.openexchange.mailfilter.ajax.Credentials;
 import com.openexchange.mailfilter.ajax.Parameter;
-import com.openexchange.mailfilter.ajax.SessionWrapper.Credentials;
 import com.openexchange.mailfilter.ajax.exceptions.OXMailfilterException;
-import com.openexchange.sessiond.exception.SessiondException;
+import com.openexchange.mailfilter.internal.MailFilterProperties;
+import com.openexchange.mailfilter.services.MailFilterServletServiceRegistry;
+import com.openexchange.session.Session;
+import com.openexchange.sessiond.SessiondException;
 import com.openexchange.tools.servlet.AjaxException;
 
 /**
- * 
+ *
  * @author <a href="mailto:marcus@open-xchange.org">Marcus Klein</a>
  */
 public abstract class AbstractRequest {
 
-    // private static final String CREDENTIALS = "credentials";
-
     private Session session;
 
     private Parameters parameters;
@@ -126,9 +128,27 @@ public abstract class AbstractRequest {
     }
 
     public Credentials getCredentials() throws SessiondException, OXMailfilterException {
-        return this.session.getCredentails();
+        final ConfigurationService config = MailFilterServletServiceRegistry.getServiceRegistry().getService(
+            ConfigurationService.class);
+        final String credsrc = config.getProperty(MailFilterProperties.Values.SIEVE_CREDSRC.property);
+        final String loginName;
+        if (MailFilterProperties.CredSrc.SESSION_FULL_LOGIN.name.equals(credsrc)) {
+            loginName = session.getLogin();
+        } else {
+            loginName = session.getLoginName();
+        }
+        final String password = session.getPassword();
+        final int userId = session.getUserId();
+        final int contextId = session.getContextId();
+
+        try {
+            final String username = getUsername();
+            return new Credentials(loginName, password, userId, contextId, username);
+        } catch (final AjaxException e) {
+            return new Credentials(loginName, password, userId, contextId);
+        }
     }
-    
+
     /**
      * @return the action
      * @throws AjaxException
@@ -149,14 +169,14 @@ public abstract class AbstractRequest {
         return retval;
     }
 
-    public interface Session {
-        Object getAttribute(String name) throws OXMailfilterException, SessiondException;
-        
-        Credentials getCredentails() throws SessiondException, OXMailfilterException;
+    private String getUsername() throws AjaxException {
+        final Parameter pUsername = Parameter.USERNAME;
+        final String username = parameters.getParameter(pUsername);
+        if (username == null) {
+            throw new AjaxException(AjaxException.Code.MISSING_PARAMETER, pUsername);
+        }
 
-        void setAttribute(String name, Object value) throws OXMailfilterException, SessiondException;
-
-        void removeAttribute(String name) throws OXMailfilterException, SessiondException;
+        return username;
     }
 
     public interface Parameters {
