@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -62,6 +62,9 @@ public class OXMailfilterException extends AbstractOXException {
 
     private static final long serialVersionUID = -4143376197058972709L;
 
+    public static final String ERR_PREFIX_INVALID_ADDRESS = "The parameter for redirect must be a valid Internet email address";
+    public static final String ERR_PREFIX_REJECTED_ADDRESS = "The Internet email address used for redirect is not allowed: ";
+
     public static enum Code {
 
     	/**
@@ -109,9 +112,9 @@ public class OXMailfilterException extends AbstractOXException {
          */
         ID_MISSING("The id is missing inside the update request or has a non integer type", Category.CODE_ERROR, 9),
         /**
-         * A server name cannot be found in the server URL
+         * A server name cannot be found in the server URL "%1$s".
          */
-        NO_SERVERNAME_IN_SERVERURL("A server name cannot be found in the server URL", Category.CODE_ERROR, 10),
+        NO_SERVERNAME_IN_SERVERURL("A server name cannot be found in the server URL \"%1$s\".", Category.CODE_ERROR, 10),
         /**
          * The login type given in the config file is not a valid one
          */
@@ -125,9 +128,9 @@ public class OXMailfilterException extends AbstractOXException {
          */
         UNSUPPORTED_ENCODING("The encoding given is not supported by Java", Category.CODE_ERROR, 13),
         /**
-         * Error in low level connection to sieve server
+         * Error in low level connection to sieve server %1$s at port %2$s
          */
-        IO_CONNECTION_ERROR("Error in low level connection to sieve server", Category.CODE_ERROR, 14),
+        IO_CONNECTION_ERROR("Error in low level connection to sieve server %1$s at port %2$s", Category.CODE_ERROR, 14),
         /**
          * Error while communicating with the sieve server %1$s at port %2$s for user %3$s in context %4$s
          */
@@ -145,15 +148,14 @@ public class OXMailfilterException extends AbstractOXException {
          */
         EMPTY_MANDATORY_FIELD("The field \"%1$s\" must have a value, but is not set", Category.USER_INPUT, 18),
         /**
-         * The configuration requests a masterpassword but none is given in the configuration file
+         * The configuration requests a master password but none is given in the configuration file
          */
-        NO_MASTERPASSWORD_SET("The configuration requests a masterpassword but none is given in the configuration file", Category.CODE_ERROR, 19),
+        NO_MASTERPASSWORD_SET("The configuration requests a master password but none is given in the configuration file", Category.CODE_ERROR, 19),
         /**
          * The passwordSource given in the config file is not a valid one
          */
         NO_VALID_PASSWORDSOURCE("The passwordSource given in the config file is not a valid one", Category.CODE_ERROR, 20);
 
-
         private final String message;
 
         private final int detailNumber;
@@ -193,6 +195,11 @@ public class OXMailfilterException extends AbstractOXException {
         super.setMessageArgs(messageArgs);
     }
 
+    public OXMailfilterException(final Category category, final int number, final String message, final Throwable cause, final Object... messageArgs) {
+        super(EnumComponent.MAIL_FILTER, category, number, message, cause);
+        super.setMessageArgs(messageArgs);
+    }
+
     public OXMailfilterException(final Component component, final Category category, final int detailNumber, final String message, final Throwable cause) {
         super(component, category, detailNumber, message, cause);
     }
