@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -53,27 +53,38 @@ import java.util.ArrayList;
 import java.util.Hashtable;
 import java.util.List;
 import javax.mail.internet.AddressException;
-import javax.mail.internet.InternetAddress;
 import org.apache.jsieve.SieveException;
 import org.json.JSONArray;
 import org.json.JSONException;
 import org.json.JSONObject;
+import com.openexchange.config.ConfigurationService;
+import com.openexchange.config.Filter;
 import com.openexchange.jsieve.commands.ActionCommand;
 import com.openexchange.jsieve.commands.IfCommand;
 import com.openexchange.jsieve.commands.Rule;
+import com.openexchange.mail.mime.QuotedInternetAddress;
 import com.openexchange.mail.utils.MailFolderUtility;
+import com.openexchange.mailfilter.ajax.exceptions.OXMailfilterException;
 import com.openexchange.mailfilter.ajax.fields.RuleFields;
 import com.openexchange.mailfilter.ajax.json.AbstractObject2JSON2Object.Mapper;
 import com.openexchange.mailfilter.ajax.json.Rule2JSON2Rule.AddFlagsActionFields;
+import com.openexchange.mailfilter.ajax.json.Rule2JSON2Rule.EnotifyActionFields;
 import com.openexchange.mailfilter.ajax.json.Rule2JSON2Rule.GeneralFields;
 import com.openexchange.mailfilter.ajax.json.Rule2JSON2Rule.MoveActionFields;
 import com.openexchange.mailfilter.ajax.json.Rule2JSON2Rule.RedirectActionFields;
 import com.openexchange.mailfilter.ajax.json.Rule2JSON2Rule.RejectActionFields;
 import com.openexchange.mailfilter.ajax.json.Rule2JSON2Rule.VacationActionFields;
+import com.openexchange.mailfilter.internal.MailFilterProperties;
+import com.openexchange.mailfilter.services.MailFilterServletServiceRegistry;
 import com.openexchange.tools.servlet.OXJSONException;
+import com.sun.mail.imap.protocol.BASE64MailboxDecoder;
+import com.sun.mail.imap.protocol.BASE64MailboxEncoder;
 
 final class ActionCommandMapper implements Mapper<Rule> {
 
+    private static final String ERR_PREFIX_INVALID_ADDRESS = OXMailfilterException.ERR_PREFIX_INVALID_ADDRESS;
+    private static final String ERR_PREFIX_REJECTED_ADDRESS = OXMailfilterException.ERR_PREFIX_REJECTED_ADDRESS;
+
     public String getAttrName() {
         return RuleFields.ACTIONCMDS;
     }
@@ -93,7 +104,7 @@ final class ActionCommandMapper implements Mapper<Rule> {
         return array;
     }
 
-    public boolean isNull(final Rule obj) {                
+    public boolean isNull(final Rule obj) {
         return (null == obj.getIfCommand());
     }
 
@@ -138,23 +149,42 @@ final class ActionCommandMapper implements Mapper<Rule> {
                 arrayList.add(Rule2JSON2Rule.createTagArg(VacationActionFields.ADDRESSES));
                 arrayList.add(Rule2JSON2Rule.JSONArrayToStringList(addresses));
             }
-            final String subject = object.getString(VacationActionFields.SUBJECT.getFieldname());
-            if (null != subject) {
+            final String subjectFieldname = VacationActionFields.SUBJECT.getFieldname();
+            if (object.has(subjectFieldname)) {
+                final String subject = object.getString(subjectFieldname);
                 arrayList.add(Rule2JSON2Rule.createTagArg(VacationActionFields.SUBJECT));
                 arrayList.add(stringToList(subject));
             }
             final String text = object.getString(VacationActionFields.TEXT.getFieldname());
             if (null == text) {
-                throw new OXJSONException(OXJSONException.Code.JSON_READ_ERROR, "Parameter " + VacationActionFields.TEXT.getFieldname() + " is missing for " + 
-                        ActionCommand.Commands.VACATION.getJsonname() + " is missing in JSON-Object. This is a required field");
+                throw new OXJSONException(
+                    OXJSONException.Code.JSON_READ_ERROR,
+                    "Parameter " + VacationActionFields.TEXT.getFieldname() + " is missing for " + ActionCommand.Commands.VACATION.getJsonname() + " is missing in JSON-Object. This is a required field");
             }
             arrayList.add(stringToList(text.replaceAll("(\r)?\n", "\r\n")));
             return new ActionCommand(ActionCommand.Commands.VACATION, arrayList);
+        } else if (ActionCommand.Commands.ENOTIFY.getJsonname().equals(id)) {
+            final ArrayList<Object> arrayList = new ArrayList<Object>();
+            final String messageFieldName = EnotifyActionFields.MESSAGE.getFieldname();
+            if (object.has(messageFieldName)) {
+                final String message = object.getString(messageFieldName);
+                arrayList.add(Rule2JSON2Rule.createTagArg(EnotifyActionFields.MESSAGE));
+                arrayList.add(stringToList(message));
+            }
+            final String method = object.getString(EnotifyActionFields.METHOD.getFieldname());
+            if (null == method) {
+                throw new OXJSONException(
+                    OXJSONException.Code.JSON_READ_ERROR,
+                    "Parameter " + EnotifyActionFields.METHOD.getFieldname() + " is missing for " + ActionCommand.Commands.ENOTIFY.getJsonname() + " is missing in JSON-Object. This is a required field");
+            }
+            arrayList.add(stringToList(method.replaceAll("(\r)?\n", "\r\n")));
+            return new ActionCommand(ActionCommand.Commands.ENOTIFY, arrayList);
         } else if (ActionCommand.Commands.ADDFLAG.getJsonname().equals(id)) {
             final JSONArray array = object.getJSONArray(AddFlagsActionFields.FLAGS);
             if (null == array) {
-                throw new OXJSONException(OXJSONException.Code.JSON_READ_ERROR, "Parameter " + AddFlagsActionFields.FLAGS + " is missing for " + 
-                        ActionCommand.Commands.ADDFLAG.getJsonname() + " is missing in JSON-Object. This is a required field");
+                throw new OXJSONException(
+                    OXJSONException.Code.JSON_READ_ERROR,
+                    "Parameter " + AddFlagsActionFields.FLAGS + " is missing for " + ActionCommand.Commands.ADDFLAG.getJsonname() + " is missing in JSON-Object. This is a required field");
             }
             final ArrayList<Object> arrayList = new ArrayList<Object>();
             arrayList.add(Rule2JSON2Rule.JSONArrayToStringList(array));
@@ -167,15 +197,20 @@ final class ActionCommandMapper implements Mapper<Rule> {
     private ActionCommand createOneParameterActionCommand(final JSONObject object, final String parameter, final ActionCommand.Commands command) throws JSONException, SieveException, OXJSONException {
         final String stringparam = getString(object, parameter, command.getCommandname());
         if (null == stringparam) {
-            throw new JSONException("The parameter " + parameter + " is missing for action command " + 
-                    command.getCommandname() + ".");
+            throw new JSONException("The parameter " + parameter + " is missing for action command " + command.getCommandname() + ".");
         }
         if (ActionCommand.Commands.REDIRECT.equals(command)) {
             // Check for valid email address here:
             try {
-                new InternetAddress(stringparam, true);
+                new QuotedInternetAddress(stringparam, true);
             } catch (final AddressException e) {
-                throw new SieveException("The parameter for redirect must be a valid email address");
+                throw new SieveException(ERR_PREFIX_INVALID_ADDRESS);
+            }
+            // And finally check of that forward address is allowed
+            final ConfigurationService service = MailFilterServletServiceRegistry.getServiceRegistry().getService(ConfigurationService.class);
+            final Filter filter;
+            if (null != service && (null != (filter = service.getFilterFromProperty("com.openexchange.mail.filter.redirectWhitelist"))) && !filter.accepts(stringparam)) {
+                throw new SieveException(ERR_PREFIX_REJECTED_ADDRESS + stringparam);
             }
         }
         return new ActionCommand(command, createArrayArray(stringparam));
@@ -184,10 +219,21 @@ final class ActionCommandMapper implements Mapper<Rule> {
     private ActionCommand createFileintoActionCommand(final JSONObject object, final String parameter, final ActionCommand.Commands command) throws JSONException, SieveException, OXJSONException {
         final String stringparam = getString(object, parameter, command.getCommandname());
         if (null == stringparam) {
-            throw new JSONException("The parameter " + parameter + " is missing for action command " + 
-                    command.getCommandname() + ".");
+            throw new JSONException("The parameter " + parameter + " is missing for action command " + command.getCommandname() + ".");
         }
-        return new ActionCommand(command, createArrayArray(MailFolderUtility.prepareMailFolderParam(stringparam).getFullname()));
+
+        final ConfigurationService config = MailFilterServletServiceRegistry.getServiceRegistry().getService(ConfigurationService.class);
+        final String encodingProperty = config.getProperty(MailFilterProperties.Values.USE_UTF7_FOLDER_ENCODING.property);
+        final boolean useUTF7Encoding = Boolean.parseBoolean(encodingProperty);
+
+        final String folderName;
+        if (useUTF7Encoding) {
+            folderName = BASE64MailboxEncoder.encode(MailFolderUtility.prepareMailFolderParam(stringparam).getFullname());
+        } else {
+            folderName = MailFolderUtility.prepareMailFolderParam(stringparam).getFullname();
+        }
+
+        return new ActionCommand(command, createArrayArray(folderName));
     }
 
     private ArrayList<Object> createArrayArray(final String string) {
@@ -205,13 +251,14 @@ final class ActionCommandMapper implements Mapper<Rule> {
     }
 
     /**
-     * This method is used to create a JSON object from a TestCommand. It is done this way because a separate
-     * converter class would have to do the check for the right TestCommand for each id.
-     * 
+     * This method is used to create a JSON object from a TestCommand. It is done this way because a separate converter class would have to
+     * do the check for the right TestCommand for each id.
+     *
      * @param tmp the JSONObject into which the values are written
      * @param actionCommand the TestCommand itself
      * @throws JSONException
      */
+    @SuppressWarnings("unchecked")
     private void createJSONFromActionCommand(final JSONObject tmp, final ActionCommand actionCommand) throws JSONException {
         if (null == actionCommand) {
             return;
@@ -245,29 +292,54 @@ final class ActionCommandMapper implements Mapper<Rule> {
                 if (null != subject) {
                     tmp.put(VacationActionFields.SUBJECT.getFieldname(), subject.get(0));
                 }
-                tmp.put(VacationActionFields.TEXT.getFieldname(), ((List<String>)arguments.get(arguments.size() - 1)).get(0));
+                tmp.put(VacationActionFields.TEXT.getFieldname(), ((List<String>) arguments.get(arguments.size() - 1)).get(0));
+            } else if (ActionCommand.Commands.ENOTIFY.equals(actionCommand.getCommand())) {
+                tmp.put(GeneralFields.ID, ActionCommand.Commands.ENOTIFY.getJsonname());
+                final Hashtable<String, List<String>> tagarguments = actionCommand.getTagarguments();
+                final List<String> message = tagarguments.get(EnotifyActionFields.MESSAGE.getTagname());
+                if (null != message) {
+                    tmp.put(EnotifyActionFields.MESSAGE.getFieldname(), message.get(0));
+                }
+                tmp.put(EnotifyActionFields.METHOD.getFieldname(), ((List<String>) arguments.get(arguments.size() - 1)).get(0));
             } else if (ActionCommand.Commands.ADDFLAG.equals(actionCommand.getCommand())) {
                 tmp.put(GeneralFields.ID, ActionCommand.Commands.ADDFLAG.getJsonname());
-                tmp.put(AddFlagsActionFields.FLAGS, (List<String>)arguments.get(0));
+                tmp.put(AddFlagsActionFields.FLAGS, (List<String>) arguments.get(0));
             }
         }
     }
 
+    @SuppressWarnings("unchecked")
     private void createOneParameterJSON(final JSONObject tmp, final ArrayList<Object> arguments, final com.openexchange.jsieve.commands.ActionCommand.Commands command, final String field) throws JSONException {
         tmp.put(GeneralFields.ID, command.getJsonname());
-        tmp.put(field, ((List<String>)arguments.get(0)).get(0));
+        tmp.put(field, ((List<String>) arguments.get(0)).get(0));
     }
 
+    @SuppressWarnings("unchecked")
     private void createFileintoJSON(final JSONObject tmp, final ArrayList<Object> arguments, final com.openexchange.jsieve.commands.ActionCommand.Commands command, final String field) throws JSONException {
         tmp.put(GeneralFields.ID, command.getJsonname());
-        tmp.put(field, MailFolderUtility.prepareFullname(0, ((List<String>)arguments.get(0)).get(0)));
+
+        final ConfigurationService config = MailFilterServletServiceRegistry.getServiceRegistry().getService(ConfigurationService.class);
+        final String encodingProperty = config.getProperty(MailFilterProperties.Values.USE_UTF7_FOLDER_ENCODING.property);
+        final boolean useUTF7Encoding = Boolean.parseBoolean(encodingProperty);
+
+        final String folderName;
+        if (useUTF7Encoding) {
+            folderName = BASE64MailboxDecoder.decode(((List<String>) arguments.get(0)).get(0));
+        } else {
+            folderName = ((List<String>) arguments.get(0)).get(0);
+        }
+
+        tmp.put(field, MailFolderUtility.prepareFullname(0, folderName));
     }
 
     private String getString(final JSONObject jobj, final String value, final String component) throws OXJSONException {
         try {
             return jobj.getString(value);
         } catch (final JSONException e) {
-            throw new OXJSONException(OXJSONException.Code.JSON_READ_ERROR, e, "Error while reading ActionCommand " + component + ": " + e.getMessage());
+            throw new OXJSONException(
+                OXJSONException.Code.JSON_READ_ERROR,
+                e,
+                "Error while reading ActionCommand " + component + ": " + e.getMessage());
         }
     }
-}
\ No newline at end of file
+}
