@@ -27,7 +27,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -47,11 +47,15 @@
  */
 package com.openexchange.mailfilter.ajax.json;
 
+import java.text.ParseException;
+import java.text.SimpleDateFormat;
 import java.util.ArrayList;
 import java.util.Collections;
+import java.util.Date;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.TimeZone;
 
 
 import org.apache.jsieve.NumberArgument;
@@ -74,6 +78,7 @@ import com.openexchange.tools.servlet.OXJSONException;
  * @author cutmasta
  *
  */
+@SuppressWarnings("unchecked")
 public class Rule2JSON2Rule extends AbstractObject2JSON2Object<Rule> {
 
     private final static String[] RULE_FIELDS_LIST = {
@@ -114,6 +119,12 @@ public class Rule2JSON2Rule extends AbstractObject2JSON2Object<Rule> {
         final static String VALUES = "values";
     }
 
+    private static class CurrentDateTestFields {
+        final static String COMPARISON = "comparison";
+        final static String DATEPART = "datepart";
+        final static String DATEVALUE = "datevalue";
+    }
+
     private static class AllofOrAnyOfTestFields {
         final static String TESTS = "tests";
     }
@@ -130,16 +141,45 @@ public class Rule2JSON2Rule extends AbstractObject2JSON2Object<Rule> {
         final static String TEXT = "text";
     }
 
+    enum EnotifyActionFields {
+        MESSAGE("message",":message"),
+        METHOD("method",null);
+
+        private final String fieldname;
+
+        private final String tagname;
+
+        private EnotifyActionFields(final String fieldname, final String tagname) {
+            this.fieldname = fieldname;
+            this.tagname = tagname;
+        }
+
+        /**
+         * @return the fieldname
+         */
+        public final String getFieldname() {
+            return fieldname;
+        }
+
+        /**
+         * @return the tagname
+         */
+        public final String getTagname() {
+            return tagname;
+        }
+
+    }
+    
     enum VacationActionFields {
         DAYS("days",":days"),
         ADDRESSES("addresses",":addresses"),
         SUBJECT("subject",":subject"),
         TEXT("text",null);
-        
+
         private final String fieldname;
-        
+
         private final String tagname;
-        
+
         private VacationActionFields(final String fieldname, final String tagname) {
             this.fieldname = fieldname;
             this.tagname = tagname;
@@ -158,52 +198,58 @@ public class Rule2JSON2Rule extends AbstractObject2JSON2Object<Rule> {
         public final String getTagname() {
             return tagname;
         }
-        
-        
+
+
     }
-    
+
     static class AddFlagsActionFields {
         final static String FLAGS = "flags";
     }
-    
-    public Rule2JSON2Rule() {      
+
+    public Rule2JSON2Rule() {
         super();
     }
-    
+
     @Override
-	protected Mapper<Rule>[] allMapper() {        
+	protected Mapper<Rule>[] allMapper() {
         return mappers;
     }
-    
+
     @Override
-	protected Rule createObject() {       
+	protected Rule createObject() {
         return new Rule();
     }
-    
+
     @Override
-	protected String[] getListFields() {        
+	protected String[] getListFields() {
         return RULE_FIELDS_LIST;
     }
-    
+
     private final static Map<String, Mapper<Rule>> attr2Mapper;
 
     @Override
 	protected Mapper<Rule> getMapper(final String attrName) {
         return attr2Mapper.get(attrName);
     }
-    
+
     static TagArgument createTagArg(final VacationActionFields fields) {
         final Token token = new Token();
         token.image = fields.getTagname();
         return new TagArgument(token);
     }
 
+    static TagArgument createTagArg(final EnotifyActionFields fields) {
+        final Token token = new Token();
+        token.image = fields.getTagname();
+        return new TagArgument(token);
+    }
+    
     public static TagArgument createTagArg(final String string) {
         final Token token = new Token();
         token.image = ":" + string;
         return new TagArgument(token);
     }
-    
+
     static List<String> JSONArrayToStringList(final JSONArray jarray) throws JSONException {
         final ArrayList<String> retval = new ArrayList<String>(jarray.length());
         for (int i = 0; i < jarray.length(); i++) {
@@ -211,35 +257,55 @@ public class Rule2JSON2Rule extends AbstractObject2JSON2Object<Rule> {
         }
         return retval;
     }
-    
+
+    static List<String> JSONDateArrayToStringList(final JSONArray jarray) throws JSONException {
+        final ArrayList<String> retval = new ArrayList<String>(jarray.length());
+        for (int i = 0; i < jarray.length(); i++) {
+            retval.add(convertJSONDate2Sieve(jarray.getString(i)));
+        }
+        return retval;
+    }
+
+    private static String convertJSONDate2Sieve(String string) throws JSONException {
+        try {
+            final Date date = new Date(Long.parseLong(string));
+            final SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
+            df.setTimeZone(TimeZone.getTimeZone("UTC"));
+            return df.format(date);
+        } catch (NumberFormatException e) {
+            throw new JSONException("Date field \"" + string + "\" is no date value");
+        }
+
+    }
+
     static NumberArgument createNumberArg(final String string) {
         final Token token = new Token();
         token.image = string;
         return new NumberArgument(token);
     }
 
-    private static final Mapper<Rule>[] mappers = new Mapper[] { 
+    private static final Mapper<Rule>[] mappers = new Mapper[] {
         new IDMapper(),
-        
+
         new Mapper<Rule>() {
-            
+
             public String getAttrName() {
                 return RuleFields.POSITION;
             }
-            
+
             public Object getAttribute(final Rule obj) throws JSONException {
                 return Integer.valueOf(obj.getPosition());
             }
-            
+
             public boolean isNull(final Rule obj) {
                 return -1 == obj.getPosition();
             }
-            
-            public void setAttribute(final Rule obj, final Object attr) throws JSONException {                
+
+            public void setAttribute(final Rule obj, final Object attr) throws JSONException {
                 obj.setPosition(((Integer)attr).intValue());
             }
         },
-        
+
         new Mapper<Rule>() {
 
             public String getAttrName() {
@@ -255,11 +321,11 @@ public class Rule2JSON2Rule extends AbstractObject2JSON2Object<Rule> {
 				return JSONObject.NULL;
             }
 
-            public boolean isNull(final Rule obj) {                
+            public boolean isNull(final Rule obj) {
                 return false;
             }
 
-            public void setAttribute(final Rule obj, final Object attr) throws JSONException {                
+            public void setAttribute(final Rule obj, final Object attr) throws JSONException {
                 final RuleComment ruleComment = obj.getRuleComment();
                 if (null != ruleComment) {
                     ruleComment.setRulename((String)attr);
@@ -268,7 +334,7 @@ public class Rule2JSON2Rule extends AbstractObject2JSON2Object<Rule> {
                 }
             }
         },
-        
+
         new Mapper<Rule>() {
 
             public String getAttrName() {
@@ -279,7 +345,7 @@ public class Rule2JSON2Rule extends AbstractObject2JSON2Object<Rule> {
                 return Boolean.valueOf(!obj.isCommented());
             }
 
-            public boolean isNull(final Rule obj) {                
+            public boolean isNull(final Rule obj) {
                 return false;
             }
 
@@ -287,7 +353,7 @@ public class Rule2JSON2Rule extends AbstractObject2JSON2Object<Rule> {
                 obj.setCommented(!((Boolean) attr).booleanValue());
             }
         },
-        
+
         new Mapper<Rule>() {
 
             public String getAttrName() {
@@ -305,7 +371,7 @@ public class Rule2JSON2Rule extends AbstractObject2JSON2Object<Rule> {
                 return tmp;
             }
 
-            public boolean isNull(final Rule obj) {                
+            public boolean isNull(final Rule obj) {
                 return false;
             }
 
@@ -338,7 +404,7 @@ public class Rule2JSON2Rule extends AbstractObject2JSON2Object<Rule> {
                 return tmp;
             }
 
-            public boolean isNull(final Rule obj) {                
+            public boolean isNull(final Rule obj) {
                 return (null == obj.getTestCommand());
             }
 
@@ -398,6 +464,28 @@ public class Rule2JSON2Rule extends AbstractObject2JSON2Object<Rule> {
                     }
                     argList.add(JSONArrayToStringList(getJSONArray(jobj, AddressEnvelopeAndHeaderTestFields.VALUES, id)));
                     return new TestCommand(TestCommand.Commands.BODY, argList, new ArrayList<TestCommand>());
+                } else if (TestCommand.Commands.CURRENTDATE.getCommandname().equals(id)) {
+                    final List<Object> argList = new ArrayList<Object>();
+                    final String comparison = getString(jobj, CurrentDateTestFields.COMPARISON, id);
+                    if ("is".equals(comparison)) {
+                        argList.add(createTagArg(comparison));
+                    } else if ("ge".equals(comparison)) {
+                        argList.add(createTagArg("value"));
+                        argList.add(getArrayFromString("ge"));
+                    } else if ("le".equals(comparison)) {
+                        argList.add(createTagArg("value"));
+                        argList.add(getArrayFromString("le"));
+                    } else {
+                        throw new OXJSONException(OXJSONException.Code.JSON_READ_ERROR, "Currentdate rule: The comparison \"" + comparison + "\" is not a valid comparison");
+                    }
+                    final String datepart = getString(jobj, CurrentDateTestFields.DATEPART, id);
+                    if ("date".equals(datepart)) {
+                        argList.add(getArrayFromString(datepart));
+                    } else {
+                        throw new OXJSONException(OXJSONException.Code.JSON_READ_ERROR, "Currentdate rule: The datepart \"" + datepart + "\" is not a valid datepart");
+                    }
+                    argList.add(JSONDateArrayToStringList(getJSONArray(jobj, CurrentDateTestFields.DATEVALUE, id)));
+                    return new TestCommand(TestCommand.Commands.CURRENTDATE, argList, new ArrayList<TestCommand>());
                 } else if (TestCommand.Commands.ALLOF.getCommandname().equals(id)) {
                     return createAllofOrAnyofTestCommand(jobj, id, TestCommand.Commands.ALLOF);
                 } else if (TestCommand.Commands.ANYOF.getCommandname().equals(id)) {
@@ -407,6 +495,12 @@ public class Rule2JSON2Rule extends AbstractObject2JSON2Object<Rule> {
                 }
             }
 
+            private List<String> getArrayFromString(String string) {
+                final List<String> retval = new ArrayList<String>();
+                retval.add(string);
+                return retval;
+            }
+
             private TestCommand createAllofOrAnyofTestCommand(final JSONObject jobj, final String id, final Commands command) throws JSONException, SieveException, OXJSONException {
                 final JSONArray jarray = getJSONArray(jobj, AllofOrAnyOfTestFields.TESTS, id);
                 final ArrayList<TestCommand> commandlist = new ArrayList<TestCommand>(jarray.length());
@@ -416,11 +510,11 @@ public class Rule2JSON2Rule extends AbstractObject2JSON2Object<Rule> {
                 }
                 return new TestCommand(command, new ArrayList<Object>(), commandlist);
             }
-            
+
             /**
              * This method is used to create a JSON object from a TestCommand. It is done this way because a separate
              * converter class would have to do the check for the right TestCommand for each id.
-             * 
+             *
              * @param tmp the JSONObject into which the values are written
              * @param testCommand the TestCommand itself
              * @throws JSONException
@@ -467,6 +561,21 @@ public class Rule2JSON2Rule extends AbstractObject2JSON2Object<Rule> {
                             tmp.put(BodyTestFields.EXTENSIONSVALUE, JSONObject.NULL);
                             tmp.put(BodyTestFields.VALUES, new JSONArray((List)testCommand.getArguments().get(2)));
                         }
+                    } else if (TestCommand.Commands.CURRENTDATE.equals(testCommand.getCommand())) {
+                        tmp.put(GeneralFields.ID, TestCommand.Commands.CURRENTDATE.getCommandname());
+                        final String comparison = testCommand.getMatchtype().substring(1);
+                        if ("value".equals(comparison)) {
+                            tmp.put(CurrentDateTestFields.COMPARISON, ((List)testCommand.getArguments().get(testCommand.getTagarguments().size())).get(0));
+                        } else {
+                            tmp.put(CurrentDateTestFields.COMPARISON, comparison);
+                        }
+                        final List value = (List)testCommand.getArguments().get(testCommand.getArguments().size()-2);
+                        tmp.put(CurrentDateTestFields.DATEPART, value.get(0));
+                        if ("date".equals(value.get(0))) {
+                            tmp.put(CurrentDateTestFields.DATEVALUE, getJSONDateArray((List)testCommand.getArguments().get(testCommand.getArguments().size()-1)));
+                        } else {
+                            tmp.put(CurrentDateTestFields.DATEVALUE, new JSONArray((List)testCommand.getArguments().get(testCommand.getArguments().size()-1)));
+                        }
                     } else if (TestCommand.Commands.ALLOF.equals(testCommand.getCommand())) {
                         createAllofOrAnyofObjects(tmp, testCommand, TestCommand.Commands.ALLOF);
                     } else if (TestCommand.Commands.ANYOF.equals(testCommand.getCommand())) {
@@ -475,6 +584,22 @@ public class Rule2JSON2Rule extends AbstractObject2JSON2Object<Rule> {
                 }
             }
 
+            private JSONArray getJSONDateArray(final List<String> collection) throws JSONException {
+                final SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd");
+                df.setTimeZone(TimeZone.getTimeZone("UTC"));
+                final JSONArray retval = new JSONArray();
+                for (final String part : collection) {
+                    Date parse;
+                    try {
+                        parse = df.parse(part);
+                        retval.put(parse.getTime());
+                    } catch (ParseException e) {
+                        throw new JSONException("Error while parsing date from string \"" + part + "\"");
+                    }
+                }
+                return retval;
+            }
+
             private void createAllofOrAnyofObjects(final JSONObject tmp, final TestCommand testCommand, final Commands command) throws JSONException {
                 tmp.put(GeneralFields.ID, command.getCommandname());
                 final JSONArray array = new JSONArray();
@@ -493,7 +618,7 @@ public class Rule2JSON2Rule extends AbstractObject2JSON2Object<Rule> {
                 argList.add(JSONArrayToStringList(getJSONArray(jobj, AddressEnvelopeAndHeaderTestFields.VALUES, command.getCommandname())));
                 return new TestCommand(command, argList, new ArrayList<TestCommand>());
             }
-            
+
             private JSONArray getJSONArray(final JSONObject jobj, final String value, final String component) throws OXJSONException {
                 try {
                     return jobj.getJSONArray(value);
@@ -501,7 +626,7 @@ public class Rule2JSON2Rule extends AbstractObject2JSON2Object<Rule> {
                     throw new OXJSONException(OXJSONException.Code.JSON_READ_ERROR, e, "Error while reading TestCommand " + component + ": " + e.getMessage());
                 }
             }
-            
+
             private String getString(final JSONObject jobj, final String value, final String component) throws OXJSONException {
                 try {
                     return jobj.getString(value);
@@ -510,9 +635,9 @@ public class Rule2JSON2Rule extends AbstractObject2JSON2Object<Rule> {
                 }
             }
         },
-        
+
         new ActionCommandMapper(),
-        
+
         new Mapper<Rule>() {
 
             public String getAttrName() {
@@ -530,31 +655,31 @@ public class Rule2JSON2Rule extends AbstractObject2JSON2Object<Rule> {
             public void setAttribute(final Rule obj, final Object attr) throws JSONException {
                 obj.setText((String)attr);
             }
-            
+
         },
 
         new Mapper<Rule>() {
-            
+
             public String getAttrName() {
                 return RuleFields.ERRORMSG;
             }
-            
+
             public Object getAttribute(final Rule obj) throws JSONException {
                 return obj.getErrormsg();
             }
-            
+
             public boolean isNull(final Rule obj) {
                 return null == obj.getErrormsg();
             }
-            
+
             public void setAttribute(final Rule obj, final Object attr) throws JSONException {
                 obj.setErrormsg((String)attr);
             }
-            
+
         }
-        
+
     };
-    
+
     static {
         final Map<String, Mapper<Rule>> tmp = new HashMap<String, Mapper<Rule>>();
         for (final Mapper<Rule> mapper : mappers) {
