@@ -28,7 +28,7 @@
  *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
  *    given Attribution for the derivative code and a license granting use.
  *
- *     Copyright (C) 2004-2010 Open-Xchange, Inc.
+ *     Copyright (C) 2004-2012 Open-Xchange, Inc.
  *     Mail: info@open-xchange.com
  *
  *
@@ -48,12 +48,19 @@
  */
 package com.openexchange.mailfilter.osgi;
 
+import java.util.Dictionary;
+import java.util.Hashtable;
+import java.util.Map;
 import java.util.Properties;
 import java.util.concurrent.atomic.AtomicBoolean;
 import org.osgi.framework.ServiceRegistration;
+import org.osgi.service.event.Event;
+import org.osgi.service.event.EventConstants;
+import org.osgi.service.event.EventHandler;
 import org.osgi.service.http.HttpService;
 import com.openexchange.config.ConfigurationService;
 import com.openexchange.groupware.settings.PreferencesItemService;
+import com.openexchange.mailfilter.ajax.actions.MailfilterAction;
 import com.openexchange.mailfilter.ajax.exceptions.OXMailfilterException;
 import com.openexchange.mailfilter.ajax.exceptions.OXMailfilterException.Code;
 import com.openexchange.mailfilter.internal.MailFilterPreferencesItem;
@@ -62,16 +69,20 @@ import com.openexchange.mailfilter.internal.MailFilterServletInit;
 import com.openexchange.mailfilter.services.MailFilterServletServiceRegistry;
 import com.openexchange.server.osgiservice.DeferredActivator;
 import com.openexchange.server.osgiservice.ServiceRegistry;
+import com.openexchange.session.Session;
+import com.openexchange.sessiond.SessiondEventConstants;
 import com.openexchange.sessiond.SessiondService;
 
 public class Activator extends DeferredActivator {
-    
-    private static final org.apache.commons.logging.Log LOG = org.apache.commons.logging.LogFactory.getLog(Activator.class);
+
+    private static final org.apache.commons.logging.Log LOG = com.openexchange.log.Log.valueOf(org.apache.commons.logging.LogFactory.getLog(Activator.class));
 
     private final AtomicBoolean started;
 
     private ServiceRegistration serviceRegistration;
 
+    private ServiceRegistration handlerRegistration;
+
     /**
      * Initializes a new {@link MailFilterServletActivator}
      */
@@ -80,11 +91,9 @@ public class Activator extends DeferredActivator {
         started = new AtomicBoolean();
     }
 
-    private static final Class<?>[] NEEDED_SERVICES = { ConfigurationService.class, HttpService.class, SessiondService.class };
-
     @Override
     protected Class<?>[] getNeededServices() {
-        return NEEDED_SERVICES;
+        return new Class<?>[] { ConfigurationService.class, HttpService.class, SessiondService.class };
     }
 
 
@@ -136,9 +145,36 @@ public class Activator extends DeferredActivator {
             }
 
             MailFilterServletInit.getInstance().start();
-            
+
             checkConfigfile();
 
+            {
+                final EventHandler eventHandler = new EventHandler() {
+
+                    public void handleEvent(final Event event) {
+                        final String topic = event.getTopic();
+                        if (SessiondEventConstants.TOPIC_REMOVE_SESSION.equals(topic)) {
+                            handleDroppedSession((Session) event.getProperty(SessiondEventConstants.PROP_SESSION));
+                        } else if (SessiondEventConstants.TOPIC_REMOVE_CONTAINER.equals(topic) || SessiondEventConstants.TOPIC_REMOVE_DATA.equals(topic)) {
+                            @SuppressWarnings("unchecked")
+                            final Map<String, Session> map = (Map<String, Session>) event.getProperty(SessiondEventConstants.PROP_CONTAINER);
+                            for (final Session session : map.values()) {
+                                handleDroppedSession(session);
+                            }
+                        }
+                    }
+
+                    private void handleDroppedSession(final Session session) {
+                        if (null == getService(SessiondService.class).getAnyActiveSessionForUser(session.getUserId(), session.getContextId())) {
+                            MailfilterAction.removeFor(session);
+                        }
+                    }
+                };
+                final Dictionary<String, Object> dict = new Hashtable<String, Object>(1);
+                dict.put(EventConstants.EVENT_TOPIC, SessiondEventConstants.getAllTopics());
+                handlerRegistration = context.registerService(EventHandler.class.getName(), eventHandler, dict);
+            }
+
             serviceRegistration = context.registerService(PreferencesItemService.class.getName(), new MailFilterPreferencesItem(), null);
         } catch (final Throwable t) {
             throw t instanceof Exception ? (Exception) t : new Exception(t);
@@ -149,6 +185,10 @@ public class Activator extends DeferredActivator {
     @Override
     public void stopBundle() throws Exception {
         try {
+            if (null != handlerRegistration) {
+                handlerRegistration.unregister();
+                handlerRegistration = null;
+            }
             if (null != serviceRegistration) {
                 serviceRegistration.unregister();
                 serviceRegistration = null;
@@ -170,7 +210,7 @@ public class Activator extends DeferredActivator {
 
     /**
      * This method checks for a valid configfile and throws and exception if now configfile is there or one of the properties is missing
-     * @throws Exception 
+     * @throws Exception
      */
     // protected to be able to test this
     protected static void checkConfigfile() throws Exception {
@@ -180,7 +220,7 @@ public class Activator extends DeferredActivator {
             throw new Exception("No configfile found for mailfilter bundle");
         }
         for (final MailFilterProperties.Values type : MailFilterProperties.Values.values()) {
-            if (type.needed && null == file.getProperty(type.property)) {
+            if (null == file.getProperty(type.property)) {
                 throw new Exception("Property for mailfilter not found: " + type.property);
             }
         }
@@ -189,7 +229,7 @@ public class Activator extends DeferredActivator {
         } catch (final NumberFormatException e) {
             throw new Exception("Property " + MailFilterProperties.Values.SIEVE_CONNECTION_TIMEOUT.property + " is no integer value");
         }
-        
+
         final String passwordsrc = config.getProperty(MailFilterProperties.Values.SIEVE_PASSWORDSRC.property);
         if (null != passwordsrc) {
             if (MailFilterProperties.PasswordSource.GLOBAL.name.equals(passwordsrc)) {
@@ -200,6 +240,8 @@ public class Activator extends DeferredActivator {
             } else if (!MailFilterProperties.PasswordSource.SESSION.name.equals(passwordsrc)) {
                 throw new OXMailfilterException(Code.NO_VALID_PASSWORDSOURCE);
             }
+        } else {
+            throw new OXMailfilterException(Code.NO_VALID_PASSWORDSOURCE);
         }
 
     }
